/***************************************************************************
                          chubsearch.h  -  description
                             -------------------
    begin                : Fri Mar 15 2002
    copyright            : (C) 2002-2003 by Mathias Kster
    email                : mathen@ketelhot.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CHUBSEARCH_H
#define CHUBSEARCH_H

/**
  *@author Mathias Kster
  */

#include <time.h>

#include <dclib/dcos.h>
#include <dclib/cstring.h>
#include <dclib/cconnection.h>

typedef enum eHubSearchType {
	ehstNONE=0,
	ehstLOCAL,
	ehstGLOBAL,
	ehstWAITTIMEOUT,
	ehstSTOP
} eHubSearchType;

typedef enum eHubSearchHubs {
	ehshCONNECTEDSINGLE=0,
	ehshCONNECTEDALL,
	ehshPUBLIC,
	ehshBOOKMARK
} eHubSearchHubs;

class CClient;

class DLL_EXPORT sHubSearchClient : public CObject {
public:
	/** */
	CClient * Client;
	/** */
	bool bRemove;
	/** */
	time_t timeHubTimeout;
};

class CStringList;

class DLL_EXPORT CHubSearch : public CConnection {
public:
	/** */
	CHubSearch();
	/** */
	virtual ~CHubSearch();

	/** callback function */
	virtual int DC_CallBack( CObject * ) { return -1; };
	/** */
	virtual void DataAvailable( const char * buffer, int len );
	/** */
	virtual void ConnectionState( eConnectionState state );

	/** */
	int StartSearch( eHubSearchHubs hubs, CList<CObject> * searchlist, CString s = "" );
	/** */
	void StopSearch();

	/** */
	int HandleMessage( char * c, int len );
	/** */
	int DC_ClientCallBack( CObject * Client, CObject * Object );
	/** */
	int DC_ManagerCallBack( CObject *, CObject * );

protected:
	/** */
	eHubSearchType ehSearchType;
	/** store clients */
	CList<CObject> * pHubSearchClientList;
	/** */
	CStringList *pHubServerList;
	/** */
	int iMaxThreads;
	/** */
	time_t tStartTime;
	/** */
	long iCurrentHubServer;

private:
	/** */
	void SendDebug( CString s );
	/** */
	void SendSearchResult( CObject * object );
	/** */
	void CheckClient();
	/** mark client as removeable */
	void RemoveClient( CClient * Client );
	/** create new client connection */
	void NewClient();
	/** send the search to this client */
	void SendSearch( CClient * Client );
	/** send the search to this client */
	int SendSearch( CString s );
	/** */
	CString GetSearchString( CMessageSearchFile * msg );
	/** */
	void UpdateClients();

	/** */
	_CCallback * m_pCallback;

	/** */
	CString sBuffer;
	/** */
	CThread ClientThread;
	/** */
	CThread HubSearchClientListThread;
	/** */
	CString * sCurrentServer;
	/** */
	CList<CObject> * m_pSearchList;
	/** */
	bool m_bHandleUserList;
};

#endif
